@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dac3026.c,v 1.52 2000/01/21 01:12:20 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dac3026.c,v 1.55 2000/11/02 02:51:19 dawes Exp $ */
 /*
  * Copyright 1994 by Robin Cutshaw <robin@XFree86.org>
  *
@@ -50,12 +50,10 @@
 #include "mga_bios.h"
 #include "mga_reg.h"
 #include "mga.h"
+#include "mga_macros.h"
 
 #include "xf86DDC.h"
 
-/* Set to 1 if you want to set MCLK from XF86Config - AT YOUR OWN RISK! */
-#define MCLK_FROM_XCONFIG 0
-
 /*
  * Only change these bits in the Option register.  Make sure that the
  * vgaioen bit is never in this mask because it is controlled elsewhere
@@ -716,16 +714,7 @@ MGA3026Init(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	   but that's no big deal since we will Restore right after
 	   this function */
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
-	MGATi3026SetMCLK( pScrn, MGAdac->MemoryClock );
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-
-#endif
+	MGA_NOT_HAL(MGATi3026SetMCLK(pScrn, MGAdac->MemoryClock));
 
 #ifdef DEBUG		
 	ErrorF("%6ld: %02X %02X %02X	%02X %02X %02X	%08lX\n", mode->Clock,
@@ -763,10 +752,7 @@ MGA3026Restore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	pciSetBitsLong(pMga->PciTag, PCI_OPTION_REG, OPTION_MASK,
 		       mgaReg->Option);
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/* select pixel clock PLL as clock source */
 	outTi3026(TVP3026_CLK_SEL, 0, mgaReg->DacRegs[3]);
 	
@@ -774,10 +760,7 @@ MGA3026Restore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	outTi3026(TVP3026_PLL_ADDR, 0, 0x2A);
 	outTi3026(TVP3026_LOAD_CLK_DATA, 0, 0);
 	outTi3026(TVP3026_PIX_CLK_DATA, 0, 0);
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	
+	);	/* MGA_NOT_HAL */
 	 
 	/*
 	 * This function handles restoring the generic VGA registers.
@@ -791,10 +774,7 @@ MGA3026Restore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	 * goes here. 
 	 */
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/* program pixel clock PLL */
 	outTi3026(TVP3026_PLL_ADDR, 0, 0x00);
 	for (i = 0; i < 3; i++)
@@ -808,31 +788,20 @@ MGA3026Restore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 
 	/* set Q divider for loop clock PLL */
 	outTi3026(TVP3026_MCLK_CTL, 0, mgaReg->DacRegs[18]);
-	
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	
+	);	/* MGA_NOT_HAL */
 	
 	/* program loop PLL */
 	outTi3026(TVP3026_PLL_ADDR, 0, 0x00);
 	for (i = 3; i < 6; i++)
 		outTi3026(TVP3026_LOAD_CLK_DATA, 0, mgaReg->DacClk[i]);
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	if ((vgaReg->MiscOutReg & 0x08) && ((mgaReg->DacClk[3] & 0xC0) == 0xC0) ) {
 		/* poll until loop PLL LOCK bit is set */
 		outTi3026(TVP3026_PLL_ADDR, 0, 0x3F);
 		while ( ! (inTi3026(TVP3026_LOAD_CLK_DATA) & 0x40) );
 	}
-	
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	
+	);	/* MGA_NOT_HAL */
 
 	/*
 	 * restore other DAC registers
@@ -885,10 +854,7 @@ MGA3026Save(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 		mgaReg->ExtVga[i] = INREG8(0x1FDF);
 	}
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	outTi3026(TVP3026_PLL_ADDR, 0, 0x00);
 	for (i = 0; i < 3; i++)
 		outTi3026(TVP3026_PIX_CLK_DATA, 0, mgaReg->DacClk[i] =
@@ -898,11 +864,7 @@ MGA3026Save(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	for (i = 3; i < 6; i++)
 		outTi3026(TVP3026_LOAD_CLK_DATA, 0, mgaReg->DacClk[i] =
 					inTi3026(TVP3026_LOAD_CLK_DATA));
-	
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	
+	);	/* MGA_NOT_HAL */
 	
 	for (i = 0; i < DACREGSIZE; i++)
 		mgaReg->DacRegs[i]	 = inTi3026(MGADACregs[i]);
