@@ -2,7 +2,7 @@
  * MGA-1064, MGA-G100, MGA-G200, MGA-G400 RAMDAC driver
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dacG.c,v 1.36 2000/06/17 00:03:19 martin Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dacG.c,v 1.42 2000/12/06 15:35:20 eich Exp $ */
 
 /*
  * This is a first cut at a non-accelerated version to work with the
@@ -25,6 +25,7 @@
 #include "mga_bios.h"
 #include "mga_reg.h"
 #include "mga.h"
+#include "mga_macros.h"
 
 #include "xf86DDC.h"
 
@@ -82,12 +83,6 @@ static Bool MGAGInit(ScrnInfoPtr, DisplayModePtr);
 static void MGAGLoadPalette(ScrnInfoPtr, int, int*, LOCO*, VisualPtr);
 static Bool MGAG_i2cInit(ScrnInfoPtr pScrn);
 
-
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-/* We don't need anymore: MGAGCalcClock and MGAGSetPCLK function */
-#ifndef USEMGAHAL
 /*
  * MGAGCalcClock - Calculate the PLL settings (m, n, p, s).
  *
@@ -150,6 +145,7 @@ MGAGCalcClock ( ScrnInfoPtr pScrn, long f_out,
 		post_div_max = 7;
 		break;
 	case PCI_CHIP_MGAG100:
+	case PCI_CHIP_MGAG100_PCI:
 	case PCI_CHIP_MGAG200:
 	case PCI_CHIP_MGAG200_PCI:
 	default:
@@ -226,7 +222,6 @@ MGAGCalcClock ( ScrnInfoPtr pScrn, long f_out,
 	return f_pll;
 }
 
-
 /*
  * MGAGSetPCLK - Set the pixel (PCLK) clock.
  */
@@ -250,10 +245,6 @@ MGAGSetPCLK( ScrnInfoPtr pScrn, long f_out )
 	pReg->DacRegs[ MGA1064_PIX_PLLC_N ] = n & 0x7F;
 	pReg->DacRegs[ MGA1064_PIX_PLLC_P ] = (p & 0x07) | ((s & 0x03) << 3);
 }
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
 
 /*
  * MGAGInit 
@@ -277,13 +268,13 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	/* 0x48: */	   0,    0,    0,    0,    0,    0,    0,    0
 	};
 
+	int i, weight555 = FALSE;
 	int hd, hs, he, ht, vd, vs, ve, vt, wd;
-	int i, BppShift;
+	int BppShift;
 	MGAPtr pMga;
 	MGARegPtr pReg;
 	vgaRegPtr pVga;
 	MGAFBLayout *pLayout;
-	int weight555 = FALSE;
 	
 	pMga = MGAPTR(pScrn);
 	pReg = &pMga->ModeReg;
@@ -292,10 +283,7 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 
 	BppShift = pMga->BppShifts[(pLayout->bitsPerPixel >> 3) - 1];
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/* Allocate the DacRegs space if not done already */
 	if (pReg->DacRegs == NULL) {
 		pReg->DacRegs = xnfcalloc(DACREGSIZE, 1);
@@ -303,6 +291,7 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	for (i = 0; i < DACREGSIZE; i++) {
 	    pReg->DacRegs[i] = initDAC[i]; 
 	}
+	);	/* MGA_NOT_HAL */
 
 	switch(pMga->Chipset)
 	{
@@ -346,6 +335,9 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 		pReg->Option2 = 0x0000007;
 		break;
 	case PCI_CHIP_MGAG400:
+#ifdef USEMGAHAL
+	       MGA_HAL(break;);
+#endif
 	       if(pMga->Dac.maxPixelClock == 360000) {  /* G400 MAX */
 	           if(pMga->OverclockMem) {
 			/* 150/200  */
@@ -386,6 +378,9 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	case PCI_CHIP_MGAG200:
 	case PCI_CHIP_MGAG200_PCI:
 	default:
+#ifdef USEMGAHAL
+	       MGA_HAL(break;);
+#endif
 		if(pMga->OverclockMem) {
                      /* 143 Mhz */
 		    pReg->DacRegs[ MGA1064_SYS_PLL_M ] = 0x06;
@@ -405,6 +400,7 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 		break;
 	}
 
+	MGA_NOT_HAL(
 	/* must always have the pci retries on but rely on 
 	   polling to keep them from occuring */
 	pReg->Option &= ~0x20000000;
@@ -436,10 +432,7 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	default:
 		FatalError("MGA: unsupported depth\n");
 	}
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
+	);	/* MGA_NOT_HAL */
 		
 	/*
 	 * This will initialize all of the generic VGA registers.
@@ -520,49 +513,23 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	pVga->CRTC[21] = vd & 0xFF;
 	pVga->CRTC[22] = (vt + 1) & 0xFF;
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
-	pReg->DacRegs[ MGA1064_CURSOR_BASE_ADR_LOW ] =
-						pMga->FbCursorOffset >> 10;
-	pReg->DacRegs[ MGA1064_CURSOR_BASE_ADR_HI ]  =
-						pMga->FbCursorOffset >> 18;
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
+	MGA_NOT_HAL(pReg->DacRegs[MGA1064_CURSOR_BASE_ADR_LOW] = pMga->FbCursorOffset >> 10);
+	MGA_NOT_HAL(pReg->DacRegs[MGA1064_CURSOR_BASE_ADR_HI] = pMga->FbCursorOffset >> 18);
 	
 	if (pMga->SyncOnGreen) {
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
-	    pReg->DacRegs[ MGA1064_GEN_CTL ] &= ~0x20;
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	    
+	    MGA_NOT_HAL(pReg->DacRegs[MGA1064_GEN_CTL] &= ~0x20);
 	    pReg->ExtVga[3] |= 0x40;
 	}
 
 	/* select external clock */
 	pVga->MiscOutReg |= 0x0C; 
 
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	if (mode->Flags & V_DBLSCAN)
 		pVga->CRTC[9] |= 0x80;
-#endif
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
-	MGAGSetPCLK( pScrn, mode->Clock );
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif	       	
+	MGAGSetPCLK(pScrn, mode->Clock);
+	);	/* MGA_NOT_HAL */
 
 	/* This disables the VGA memory aperture */
 	pVga->MiscOutReg &= ~0x02;
@@ -573,6 +540,29 @@ MGAGInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
  * MGAGLoadPalette
  */
 
+static void
+MGAPaletteLoadCallback(ScrnInfoPtr pScrn)
+{
+    MGAPtr pMga = MGAPTR(pScrn);
+    MGAPaletteInfo *pal = pMga->palinfo;
+    int i;
+
+    while (!(INREG8(0x1FDA) & 0x08)); 
+
+    for(i = 0; i < 256; i++) {
+	if(pal->update) {
+	    outMGAdreg(MGA1064_WADR_PAL, i);
+            outMGAdreg(MGA1064_COL_PAL, pal->red);
+            outMGAdreg(MGA1064_COL_PAL, pal->green);
+            outMGAdreg(MGA1064_COL_PAL, pal->blue);
+	    pal->update = FALSE;
+	}
+	pal++;
+    }
+    
+    pMga->PaletteLoadCallback = NULL;
+}
+
 void MGAGLoadPalette(
     ScrnInfoPtr pScrn, 
     int numColors, 
@@ -581,17 +571,30 @@ void MGAGLoadPalette(
     VisualPtr pVisual
 ){
     MGAPtr pMga = MGAPTR(pScrn);
-    int i, index;
 
-    if(pMga->CurrentLayout.Overlay8Plus24 && (pVisual->nplanes != 8))
+    if((pMga->CurrentLayout.Overlay8Plus24) && (pVisual->nplanes != 8)) 
 	return;
 
-    for(i = 0; i < numColors; i++) {
-	index = indices[i];
-        outMGAdreg(MGA1064_WADR_PAL, index);
-        outMGAdreg(MGA1064_COL_PAL, colors[index].red);
-        outMGAdreg(MGA1064_COL_PAL, colors[index].green);
-        outMGAdreg(MGA1064_COL_PAL, colors[index].blue);
+     if(pMga->Chipset == PCI_CHIP_MGAG400){ 
+	 /* load them at the retrace in the block handler instead to 
+	    work around some problems with static on the screen */
+	while(numColors--) {
+	    pMga->palinfo[*indices].update = TRUE;
+	    pMga->palinfo[*indices].red   = colors[*indices].red;
+	    pMga->palinfo[*indices].green = colors[*indices].green;
+	    pMga->palinfo[*indices].blue  = colors[*indices].blue;
+	    indices++;
+	}
+	pMga->PaletteLoadCallback = MGAPaletteLoadCallback;
+	return;
+    } else {
+	while(numColors--) {
+            outMGAdreg(MGA1064_WADR_PAL, *indices);
+            outMGAdreg(MGA1064_COL_PAL, colors[*indices].red);
+            outMGAdreg(MGA1064_COL_PAL, colors[*indices].green);
+            outMGAdreg(MGA1064_COL_PAL, colors[*indices].blue);
+	    indices++;
+	}
     }
 }
 
@@ -634,13 +637,10 @@ MGAGRestore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	       Bool restoreFonts)
 {
 	int i;
-	CARD32 optionMask;
 	MGAPtr pMga = MGAPTR(pScrn);
+	CARD32 optionMask;
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/*
 	 * Code is needed to get things back to bank zero.
 	 */
@@ -674,11 +674,8 @@ MGAGRestore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	if (pMga->Chipset == PCI_CHIP_MGAG400)
 		pciSetBitsLong(pMga->PciTag, PCI_MGA_OPTION3, OPTION3_MASK,
 			       mgaReg->Option3);
+	);	/* MGA_NOT_HAL */
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
 	/* restore CRTCEXT regs */
 	for (i = 0; i < 6; i++)
 		OUTREG16(0x1FDE, (mgaReg->ExtVga[i] << 8) | i);
@@ -727,19 +724,12 @@ MGAGSave(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	MGAPtr pMga = MGAPTR(pScrn);
 
 	if(pMga->SecondCrtc == TRUE) return;
-
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/* Allocate the DacRegs space if not done already */
 	if (mgaReg->DacRegs == NULL) {
 		mgaReg->DacRegs = xnfcalloc(DACREGSIZE, 1);
 	}
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
+	);	/* MGA_NOT_HAL */
 
 	/*
 	 * Code is needed to get back to bank zero.
@@ -753,10 +743,7 @@ MGAGSave(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	vgaHWSave(pScrn, vgaReg, VGA_SR_MODE | (saveFonts ? VGA_SR_FONTS : 0));
 	MGAGSavePalette(pScrn, vgaReg->DAC);
 
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#ifndef USEMGAHAL
+	MGA_NOT_HAL(
 	/*
 	 * The port I/O code necessary to read in the extended registers.
 	 */
@@ -768,10 +755,7 @@ MGAGSave(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, MGARegPtr mgaReg,
 	mgaReg->Option2 = pciReadLong(pMga->PciTag, PCI_MGA_OPTION2);
 	if (pMga->Chipset == PCI_CHIP_MGAG400)
 	    mgaReg->Option3 = pciReadLong(pMga->PciTag, PCI_MGA_OPTION3);
-/*******************/
-/* ADDED BY MATROX */
-/*******************/
-#endif
+	);	/* MGA_NOT_HAL */
 
 	for (i = 0; i < 6; i++)
 	{
