@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dri.c,v 1.5 2000/06/23 23:43:44 alanh Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dri.c,v 1.14 2000/12/11 22:34:55 dawes Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -101,12 +101,10 @@ Bool MgaInitDma(ScrnInfoPtr pScrn, int prim_size)
       init.chipset = MGA_CARD_TYPE_G400;
       break;
    case PCI_CHIP_MGAG200:
+   case PCI_CHIP_MGAG200_PCI:
       init.chipset = MGA_CARD_TYPE_G200;
       break;
-   case PCI_CHIP_MGAG200_PCI:
    default:
-      xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
-                 "[drm] Direct rendering not supported on this card/chipset\n");
       return FALSE;
    }
 
@@ -148,10 +146,9 @@ MGAInitVisualConfigs(ScreenPtr pScreen)
    switch (pScrn->bitsPerPixel) {
    case 8:
    case 24:
-   case 32:
       break;
    case 16:
-      numConfigs = 4;
+      numConfigs = 8;
 
       if (!(pConfigs = (__GLXvisualConfig*)xnfcalloc(sizeof(__GLXvisualConfig),
 						     numConfigs))) {
@@ -174,14 +171,15 @@ MGAInitVisualConfigs(ScreenPtr pScreen)
       i = 0;
       depth = 1;
       for (accum = 0; accum <= 1; accum++) {
-         for (stencil = 0; stencil <= 0; stencil++) { /* no stencil for now */
-            for (db=0; db<=1; db++) {
+         for (stencil = 0; stencil <= 1; stencil++) { /* no stencil for now */
+            for (db=1; db>=0; db--) {
                pConfigs[i].vid = -1;
                pConfigs[i].class = -1;
                pConfigs[i].rgba = TRUE;
                pConfigs[i].redSize = 5;
                pConfigs[i].greenSize = 6;
                pConfigs[i].blueSize = 5;
+               pConfigs[i].alphaSize = 0;
                pConfigs[i].redMask = 0x0000F800;
                pConfigs[i].greenMask = 0x000007E0;
                pConfigs[i].blueMask = 0x0000001F;
@@ -213,7 +211,7 @@ MGAInitVisualConfigs(ScreenPtr pScreen)
                   pConfigs[i].stencilSize = 0;
                pConfigs[i].auxBuffers = 0;
                pConfigs[i].level = 0;
-               if (stencil)
+               if (stencil || accum)
                   pConfigs[i].visualRating = GLX_SLOW_VISUAL_EXT;
                else
                   pConfigs[i].visualRating = GLX_NONE_EXT;
@@ -233,6 +231,91 @@ MGAInitVisualConfigs(ScreenPtr pScreen)
          return FALSE;
       }
       break;
+
+   case 32:
+      numConfigs = 8;
+
+      if (!(pConfigs = (__GLXvisualConfig*)xnfcalloc(sizeof(__GLXvisualConfig),
+						     numConfigs))) {
+	 return FALSE;
+      }
+      if (!(pMGAConfigs = (MGAConfigPrivPtr)xnfcalloc(sizeof(MGAConfigPrivRec),
+						      numConfigs))) {
+	 xfree(pConfigs);
+	 return FALSE;
+      }
+      if (!(pMGAConfigPtrs = (MGAConfigPrivPtr*)xnfcalloc(sizeof(MGAConfigPrivPtr),
+							  numConfigs))) {
+	 xfree(pConfigs);
+	 xfree(pMGAConfigs);
+	 return FALSE;
+      }
+      for (i=0; i<numConfigs; i++) 
+	 pMGAConfigPtrs[i] = &pMGAConfigs[i];
+
+      i = 0;
+      for (accum = 0; accum <= 1; accum++) {
+         for (depth = 0; depth <= 1; depth++) { /* and stencil */
+            for (db=1; db>=0; db--) {
+               pConfigs[i].vid = -1;
+               pConfigs[i].class = -1;
+               pConfigs[i].rgba = TRUE;
+               pConfigs[i].redSize = 8;
+               pConfigs[i].greenSize = 8;
+               pConfigs[i].blueSize = 8;
+               pConfigs[i].alphaSize = 0;
+               pConfigs[i].redMask   = 0x00FF0000;
+               pConfigs[i].greenMask = 0x0000FF00;
+               pConfigs[i].blueMask  = 0x000000FF;
+               pConfigs[i].alphaMask = 0;
+               if (accum) {
+                  pConfigs[i].accumRedSize = 16;
+                  pConfigs[i].accumGreenSize = 16;
+                  pConfigs[i].accumBlueSize = 16;
+                  pConfigs[i].accumAlphaSize = 0;
+               } else {
+                  pConfigs[i].accumRedSize = 0;
+                  pConfigs[i].accumGreenSize = 0;
+                  pConfigs[i].accumBlueSize = 0;
+                  pConfigs[i].accumAlphaSize = 0;
+               }
+               if (db)
+                  pConfigs[i].doubleBuffer = TRUE;
+               else
+                  pConfigs[i].doubleBuffer = FALSE;
+               pConfigs[i].stereo = FALSE;
+               pConfigs[i].bufferSize = 32;
+               if (depth) {
+		     pConfigs[i].depthSize = 24;
+                     pConfigs[i].stencilSize = 8;
+               }
+               else {
+                     pConfigs[i].depthSize = 0;
+                     pConfigs[i].stencilSize = 0;
+               }
+               pConfigs[i].auxBuffers = 0;
+               pConfigs[i].level = 0;
+               if (accum)
+                  pConfigs[i].visualRating = GLX_SLOW_VISUAL_EXT;
+               else
+                  pConfigs[i].visualRating = GLX_NONE_EXT;
+               pConfigs[i].transparentPixel = 0;
+               pConfigs[i].transparentRed = 0;
+               pConfigs[i].transparentGreen = 0;
+               pConfigs[i].transparentBlue = 0;
+               pConfigs[i].transparentAlpha = 0;
+               pConfigs[i].transparentIndex = 0;
+               i++;
+            }
+         }
+      }
+      if (i != numConfigs) {
+         xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
+                    "[drm] Incorrect initialization of visuals\n");
+         return FALSE;
+      }
+      break;
+
    default:
       ;  /* unexpected bits/pixelx */
    }
@@ -250,6 +333,39 @@ static unsigned int mylog2(unsigned int n)
    return log2;
 }
 
+static unsigned long MGAParseAgpMode(ScreenPtr pScreen)
+{
+   ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+   MGAPtr pMga = MGAPTR(pScrn);
+   unsigned long mode_mask;
+
+   switch(pMga->agp_mode) {
+   case 4:
+      mode_mask = ~0x00000003;
+      break;
+   case 2:
+      if (pMga->Chipset == PCI_CHIP_MGAG200) {
+	 xf86DrvMsg(pScreen->myNum, X_INFO, 
+		    "[drm] Enabling AGP 2x pll encoding\n");
+	 OUTREG(MGAREG_AGP_PLL, AGP_PLL_agp2xpllen_enable);
+      }
+      mode_mask = ~0x00000005;
+      break;
+   default:
+   /* Default to 1X agp mode */
+   case 1:
+      if (pMga->Chipset == PCI_CHIP_MGAG200) {
+	 xf86DrvMsg(pScreen->myNum, X_INFO, 
+		    "[drm] Disabling AGP 2x pll encoding\n");
+	 OUTREG(MGAREG_AGP_PLL, AGP_PLL_agp2xpllen_disable);
+      }
+      pMga->agp_mode = 1;
+      mode_mask = ~0x00000006;
+      break;
+   }
+
+   return mode_mask;
+}
 
 Bool MGADRIScreenInit(ScreenPtr pScreen)
 {
@@ -262,6 +378,19 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
    int prim_size;
    int init_offset;
    int i;
+   unsigned long mode_mask;
+
+   switch(pMGA->Chipset) {
+   case PCI_CHIP_MGAG400:
+   case PCI_CHIP_MGAG200:
+#if 0
+   case PCI_CHIP_MGAG200_PCI:
+#endif
+      break;
+   default:
+      xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "[drm] Direct rendering only supported with G200/G400 AGP\n");
+      return FALSE;
+   }
 
    /* Check that the GLX, DRI, and DRM modules have been loaded by testing
     * for canonical symbols in each module. */
@@ -270,7 +399,7 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
    if (!xf86LoaderCheckSymbol("drmAvailable"))        return FALSE;
    if (!xf86LoaderCheckSymbol("DRIQueryVersion")) {
       xf86DrvMsg(pScreen->myNum, X_ERROR,
-                 "TDFXDRIScreenInit failed (libdri.a too old)\n");
+                 "MGADRIScreenInit failed (libdri.a too old)\n");
       return FALSE;
    }
      
@@ -286,9 +415,12 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
       }
    }
 
-   if ((pScrn->bitsPerPixel / 8) != 2) {
+   xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] bpp: %d depth: %d\n", pScrn->bitsPerPixel, pScrn->depth);
+
+   if ((pScrn->bitsPerPixel / 8) != 2 &&
+       (pScrn->bitsPerPixel / 8) != 4) {
       xf86DrvMsg(pScreen->myNum, X_INFO,
-                 "[drm] Direct Rendering only supported in 16 bpp mode\n");
+                 "[drm] Direct rendering only supported in 16 and 32 bpp modes\n");
       return FALSE;
    }
    
@@ -394,16 +526,17 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
    {
       drmVersionPtr version = drmGetVersion(pMGA->drmSubFD);
       if (version) {
-         if (version->version_major != 1 ||
+         if (version->version_major != 2 ||
              version->version_minor != 0 ||
              version->version_patchlevel < 0) {
             /* incompatible drm version */
             xf86DrvMsg(pScreen->myNum, X_ERROR,
-                       "[drm] MGADRIScreenInit failed (DRM version = %d.%d.%d, expected 1.0.x).  Disabling DRI.\n",
+                       "[drm] MGADRIScreenInit failed (DRM version = %d.%d.%d, expected 2.0.x).  Disabling DRI.\n",
                        version->version_major,
                        version->version_minor,
                        version->version_patchlevel);
-            MGADRICloseScreen(pScreen);
+/*              MGADRICloseScreen(pScreen); */
+	    
             drmFreeVersion(version);
             return FALSE;
          }
@@ -442,16 +575,17 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
       DRICloseScreen(pScreen);
       return FALSE;
    }
-   
+
+   mode_mask = MGAParseAgpMode(pScreen);
+
    pMGADRIServer->agpMode = drmAgpGetMode(pMGA->drmSubFD);
-   /* Default to 1X agp mode */
-   pMGADRIServer->agpMode &= ~0x00000002;
+   pMGADRIServer->agpMode &= mode_mask;
    if (drmAgpEnable(pMGA->drmSubFD, pMGADRIServer->agpMode) < 0) {
       xf86DrvMsg(pScreen->myNum, X_ERROR, "[drm] drmAgpEnable failed\n");
       DRICloseScreen(pScreen);
       return FALSE;
    }
-   ErrorF("[drm] drmAgpEnabled succeeded\n");
+   ErrorF("[drm] drmAgpEnabled succeeded for AGP mode %dx\n", pMGA->agp_mode);
 
    prim_size = 65536;
    init_offset = ((prim_size + pMGADRIServer->warp_ucode_size + 
@@ -513,12 +647,10 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
       pMGADRI->chipset = MGA_CARD_TYPE_G400;
       break;
    case PCI_CHIP_MGAG200:
+   case PCI_CHIP_MGAG200_PCI:
       pMGADRI->chipset = MGA_CARD_TYPE_G200;
       break;
-   case PCI_CHIP_MGAG200_PCI:
    default:
-      xf86DrvMsg(pScreen->myNum, X_ERROR,
-                "[drm] Direct rendering not supported on this card/chipset\n");
       return FALSE;
    }
    
@@ -532,20 +664,14 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
    pMGADRI->frontOffset = 0; /* pMGA->YDstOrg * (pScrn->bitsPerPixel / 8) */
    pMGADRI->backOffset = ((pScrn->virtualY + pMGA->numXAALines + 1) * 
 			  pScrn->displayWidth *
-                          (pScrn->bitsPerPixel / 8) + 4095) & ~0xFFF;
+                          pMGADRI->cpp + 4095) & ~0xFFF;
 
 
    xf86DrvMsg(pScreen->myNum, X_INFO, "[drm] calced backoffset: 0x%x\n",
               pMGADRI->backOffset);
 
 
-#if 0
-   size = 2 * pScrn->virtualX * pScrn->virtualY;
-   pMGADRI->depthOffset = (pMGADRI->backOffset + size + 4095) & ~0xFFF;
-   pMGADRI->textureOffset = pMGADRI->depthOffset + size;
-   pMGADRI->textureSize = pMGA->FbUsableSize - pMGADRI->textureOffset;
-#else
-   size = 2 * pScrn->virtualX * pScrn->virtualY;
+   size = pMGADRI->cpp * pScrn->virtualX * pScrn->virtualY;
    size += 4095;
    size &= ~4095;
    pMGADRI->depthOffset = pMGA->FbUsableSize - size;
@@ -559,7 +685,6 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
       DRICloseScreen(pScreen);
       return FALSE;
    }
-#endif
 
    pMGADRI->mAccess = pMGA->MAccess;
 
@@ -625,6 +750,25 @@ Bool MGADRIScreenInit(ScreenPtr pScreen)
 					     ->thisCard)->devnum,
 					    ((pciConfigPtr)pMGA->PciInfo
 					     ->thisCard)->funcnum);
+
+      if(!pMGADRIServer->irq && !pMGA->ReallyUseIrqZero) {
+	 xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+		    "[drm] Your graphics card has Interrupt zero"
+		    " assigned to it.\n"
+		    "This is highly unlikely so I'm disabling the DRI.\n"
+		    "If your graphics card really has Interrupt zero, please"
+		    "add the config option UseIrqZero\n"
+		    "to the device section in your XF86Config file.\n"
+		    "Please be warned that Interrupt zero is normally "
+		    "the timer interrupt on X86 systems.\n"
+		    "Using this option could make your system unusable.\n"
+		    "The more likely solution is that your graphics card has"
+		    " no interrupt assigned to it.\nPlease consult your"
+		    " system BIOS manual for instructions on how to enable "
+		    "an interrupt for your graphics card.\n");
+	 MGADRICloseScreen(pScreen);
+	 return FALSE;
+      }
       drmCtlInstHandler(pMGA->drmSubFD, pMGADRIServer->irq);
    }
 
@@ -649,22 +793,16 @@ MGADRICloseScreen(ScreenPtr pScreen)
   MGAPtr pMGA = MGAPTR(pScrn);
   MGADRIServerPrivatePtr pMGADRIServer = pMGA->DRIServerInfo;
 
-  MgaCleanupDma(pScrn);
-
+/* The DRI will automagically clean these up when driFD is closed */
   if(pMGADRIServer->agp_map) {  
-     ErrorF("Unmapped agp region\n");
      drmUnmap(pMGADRIServer->agp_map, pMGADRIServer->agpSizep);
      pMGADRIServer->agp_map = 0;
   }
   if(pMGADRIServer->agpHandle) {
-     ErrorF("Freeing agp memory\n");
-     drmAgpFree(pMGA->drmSubFD, pMGADRIServer->agpHandle);
      pMGADRIServer->agpHandle = 0;
      pMGADRIServer->agpSizep = 0;
   }
   if(pMGADRIServer->agpAcquired == TRUE) {
-     ErrorF("releasing agp module\n");
-     drmAgpRelease(pMGA->drmSubFD);
      pMGADRIServer->agpAcquired = FALSE;
   }
 
@@ -734,11 +872,13 @@ mgaGetQuiescence( ScrnInfoPtr pScrn )
 
       MgaLockUpdate(pScrn, (DRM_LOCK_QUIESCENT | DRM_LOCK_FLUSH));	
 
-      WAITFIFO(12);
+      WAITFIFO(11);
       OUTREG(MGAREG_MACCESS, pMga->MAccess);
       OUTREG(MGAREG_PITCH, pLayout->displayWidth);
-      OUTREG(MGAREG_YDSTORG, pMga->YDstOrg);
+      pMga->PlaneMask = ~0;
       OUTREG(MGAREG_PLNWT, pMga->PlaneMask);
+      pMga->BgColor = 0;
+      pMga->FgColor = 0;
       OUTREG(MGAREG_BCOL, pMga->BgColor);
       OUTREG(MGAREG_FCOL, pMga->FgColor);
       OUTREG(MGAREG_SRCORG, pMga->realSrcOrg);
@@ -766,7 +906,7 @@ mgaGetQuiescence_shared( ScrnInfoPtr pScrn )
    
    if (pMgaEnt->directRenderingEnabled) {
       MgaLockUpdate(pMgaEnt->pScrn_1, (DRM_LOCK_QUIESCENT | DRM_LOCK_FLUSH));
-      MGARestoreAccelState(pScrn);
+      pMga->RestoreAccelState(pScrn);
       xf86SetLastScrnFlag(pScrn->entityList[0], pScrn->scrnIndex);
    }
 }
