@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_video.c,v 1.15 2000/06/17 00:03:20 martin Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_video.c,v 1.21 2000/12/05 20:03:45 mvojkovi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -30,7 +30,7 @@
 
 #define TIMER_MASK      (OFF_TIMER | FREE_TIMER)
 
-#define MGA_MAX_PORTS	16
+#define MGA_MAX_PORTS	32
 
 #ifndef XvExtension
 void MGAInitVideo(ScreenPtr pScreen) {}
@@ -55,10 +55,11 @@ static int  MGAPutImage(ScrnInfoPtr, short, short, short, short, short,
 static int  MGAQueryImageAttributes(ScrnInfoPtr, int, unsigned short *, 
 			unsigned short *,  int *, int *);
 
-static void MGABlockHandler(int, pointer, pointer, pointer);
 
 static void MGAResetVideoOverlay(ScrnInfoPtr);
 
+static void MGAVideoTimerCallback(ScrnInfoPtr pScrn, Time time);
+
 
 #define MAKE_ATOM(a) MakeAtom(a, sizeof(a) - 1, TRUE)
 
@@ -73,12 +74,12 @@ void MGAInitVideo(ScreenPtr pScreen)
     int num_adaptors;
 
     if((pScrn->bitsPerPixel != 8) && !pMga->NoAccel &&
+       (pMga->SecondCrtc == FALSE) &&
        ((pMga->Chipset == PCI_CHIP_MGAG200) ||
         (pMga->Chipset == PCI_CHIP_MGAG200_PCI) ||
         (pMga->Chipset == PCI_CHIP_MGAG400))) 
     {
-
-	if((pMga->Overlay8Plus24 /* || dualhead */ || pMga->TexturedVideo) &&
+	if((pMga->Overlay8Plus24 || pMga->TexturedVideo) &&
 	   (pScrn->bitsPerPixel != 24))
         {
 	    xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Using texture video\n");
@@ -89,14 +90,9 @@ void MGAInitVideo(ScreenPtr pScreen)
 	    newAdaptor = MGASetupImageVideoOverlay(pScreen);
 	    pMga->TexturedVideo = FALSE;
 	}
-
-	if(!pMga->Overlay8Plus24 /* && !dualhead */)
+	if(!pMga->Overlay8Plus24)
 	    MGAInitOffscreenImages(pScreen);
-
-	pMga->BlockHandler = pScreen->BlockHandler;
-	pScreen->BlockHandler = MGABlockHandler;
     }
-    
 
     num_adaptors = xf86XVListGenericAdaptors(pScrn, &adaptors);
 
@@ -141,10 +137,9 @@ static XF86VideoEncodingRec DummyEncoding[2] =
  }
 };
 
-#define NUM_FORMATS_OVERLAY 3
-#define NUM_FORMATS_TEXTURE 6
+#define NUM_FORMATS 6
 
-static XF86VideoFormatRec Formats[NUM_FORMATS_TEXTURE] = 
+static XF86VideoFormatRec Formats[NUM_FORMATS] = 
 {
    {15, TrueColor}, {16, TrueColor}, {24, TrueColor},
    {15, DirectColor}, {16, DirectColor}, {24, DirectColor}
@@ -251,7 +246,7 @@ MGASetupImageVideoOverlay(ScreenPtr pScreen)
     adapt->name = "Matrox G-Series Backend Scaler";
     adapt->nEncodings = 1;
     adapt->pEncodings = &DummyEncoding[0];
-    adapt->nFormats = NUM_FORMATS_OVERLAY;
+    adapt->nFormats = NUM_FORMATS;
     adapt->pFormats = Formats;
     adapt->nPorts = 1;
     adapt->pAttributes = Attributes;
@@ -297,7 +292,7 @@ MGASetupImageVideoTexture(ScreenPtr pScreen)
     adapt->name = "Matrox G-Series Texture Engine";
     adapt->nEncodings = 1;
     adapt->pEncodings = &DummyEncoding[1];
-    adapt->nFormats = NUM_FORMATS_TEXTURE;
+    adapt->nFormats = NUM_FORMATS;
     adapt->pFormats = Formats;
     adapt->nPorts = MGA_MAX_PORTS;
     adapt->pAttributes = NULL;
@@ -444,7 +439,7 @@ MGAClipVideo(
 } 
 
 static void 
-MGAStopVideo(ScrnInfoPtr pScrn, pointer data, Bool exit)
+MGAStopVideo(ScrnInfoPtr pScrn, pointer data, Bool shutdown)
 {
   MGAPtr pMga = MGAPTR(pScrn);
   MGAPortPrivPtr pPriv = pMga->portPrivate;
@@ -453,12 +448,12 @@ MGAStopVideo(ScrnInfoPtr pScrn, pointer data, Bool exit)
 
   REGION_EMPTY(pScrn->pScreen, &pPriv->clip);   
 
-  if(exit) {
+  if(shutdown) {
      if(pPriv->videoStatus & CLIENT_VIDEO_ON)
 	OUTREG(MGAREG_BESCTL, 0);
-     if(pPriv->area) {
-	xf86FreeOffscreenArea(pPriv->area);
-	pPriv->area = NULL;
+     if(pPriv->linear) {
+	xf86FreeOffscreenLinear(pPriv->linear);
+	pPriv->linear = NULL;
      }
      pPriv->videoStatus = 0;
   } else {
@@ -597,66 +592,80 @@ MGACopyMungedData(
    int h,
    int w
 ){
-   CARD32 *dst = (CARD32*)dst1;
+   CARD32 *dst;
+   CARD8 *s1, *s2, *s3;
    int i, j;
 
-   dstPitch >>= 2;
    w >>= 1;
 
    for(j = 0; j < h; j++) {
-	for(i = 0; i < w; i++) {
-	    dst[i] = src1[i << 1] | (src1[(i << 1) + 1] << 16) |
-		     (src3[i] << 8) | (src2[i] << 24);
-	}
-	dst += dstPitch;
-	src1 += srcPitch;
-	if(j & 1) {
-	    src2 += srcPitch2;
-	    src3 += srcPitch2;
-	}
+        dst = (CARD32*)dst1;
+        s1 = src1;  s2 = src2;  s3 = src3;
+        i = w;
+        while(i > 4) {
+           dst[0] = s1[0] | (s1[1] << 16) | (s3[0] << 8) | (s2[0] << 24);
+           dst[1] = s1[2] | (s1[3] << 16) | (s3[1] << 8) | (s2[1] << 24);
+           dst[2] = s1[4] | (s1[5] << 16) | (s3[2] << 8) | (s2[2] << 24);
+           dst[3] = s1[6] | (s1[7] << 16) | (s3[3] << 8) | (s2[3] << 24);
+           dst += 4; s2 += 4; s3 += 4; s1 += 8;
+           i -= 4;
+        }
+
+        while(i--) {
+           dst[0] = s1[0] | (s1[1] << 16) | (s3[0] << 8) | (s2[0] << 24);
+           dst++; s2++; s3++;
+           s1 += 2;
+        }
+
+        dst1 += dstPitch;
+        src1 += srcPitch;
+        if(j & 1) {
+            src2 += srcPitch2;
+            src3 += srcPitch2;
+        }
    }
 }
 
 
-static FBAreaPtr
+static FBLinearPtr
 MGAAllocateMemory(
    ScrnInfoPtr pScrn,
-   FBAreaPtr area,
-   int numlines
+   FBLinearPtr linear,
+   int size
 ){
    ScreenPtr pScreen;
-   FBAreaPtr new_area;
+   FBLinearPtr new_linear;
 
-   if(area) {
-	if((area->box.y2 - area->box.y1) >= numlines) 
-	   return area;
+   if(linear) {
+	if(linear->size >= size) 
+	   return linear;
         
-        if(xf86ResizeOffscreenArea(area, pScrn->displayWidth, numlines))
-	   return area;
+        if(xf86ResizeOffscreenLinear(linear, size))
+	   return linear;
 
-	xf86FreeOffscreenArea(area);
+	xf86FreeOffscreenLinear(linear);
    }
 
    pScreen = screenInfo.screens[pScrn->scrnIndex];
 
-   new_area = xf86AllocateOffscreenArea(pScreen, pScrn->displayWidth, 
-				numlines, 0, NULL, NULL, NULL);
+   new_linear = xf86AllocateOffscreenLinear(pScreen, size, 16, 
+   						NULL, NULL, NULL);
 
-   if(!new_area) {
-	int max_w, max_h;
+   if(!new_linear) {
+	int max_size;
 
-	xf86QueryLargestOffscreenArea(pScreen, &max_w, &max_h, 0,
-			FAVOR_WIDTH_THEN_AREA, PRIORITY_EXTREME);
+	xf86QueryLargestOffscreenLinear(pScreen, &max_size, 16, 
+						PRIORITY_EXTREME);
 	
-	if((max_w < pScrn->displayWidth) || (max_h < numlines))
+	if(max_size < size)
 	   return NULL;
 
 	xf86PurgeUnlockedOffscreenAreas(pScreen);
-	new_area = xf86AllocateOffscreenArea(pScreen, pScrn->displayWidth, 
-				numlines, 0, NULL, NULL, NULL);
+	new_linear = xf86AllocateOffscreenLinear(pScreen, size, 16, 
+						NULL, NULL, NULL);
    }
 
-   return new_area;
+   return new_linear;
 }
 
 static void
@@ -734,7 +743,7 @@ MGADisplayVideoTexture(
     short drw_w, short drw_h
 ){
     MGAPtr pMga = MGAPTR(pScrn);
-    int log2w, log2h, i, incx, incy, padw, padh;
+    int log2w = 0, log2h = 0, i, incx, incy, padw, padh;
     
     pitch >>= 1;
 
@@ -818,16 +827,16 @@ MGAPutImage(
   short drw_w, short drw_h,
   int id, unsigned char* buf, 
   short width, short height, 
-  Bool sync,
+  Bool Sync,
   RegionPtr clipBoxes, pointer data
 ){
    MGAPtr pMga = MGAPTR(pScrn);
    MGAPortPrivPtr pPriv = pMga->portPrivate;
    INT32 x1, x2, y1, y2;
    unsigned char *dst_start;
-   int pitch, new_h, offset, offset2, offset3;
-   int srcPitch, srcPitch2, dstPitch;
-   int top, left, npixels, nlines;
+   int pitch, new_size, offset, offset2 = 0, offset3 = 0;
+   int srcPitch, srcPitch2 = 0, dstPitch;
+   int top, left, npixels, nlines, bpp;
    BoxRec dstBox;
    CARD32 tmp;
 
@@ -852,11 +861,12 @@ MGAPutImage(
 	dstBox.y2 -= pScrn->frameY0;
    }
 
-   pitch = pScrn->bitsPerPixel * pScrn->displayWidth >> 3;
+   bpp = pScrn->bitsPerPixel >> 3;
+   pitch = bpp * pScrn->displayWidth;
 
    dstPitch = ((width << 1) + 15) & ~15;
-   new_h = ((dstPitch * height) + pitch - 1) / pitch;
-
+   new_size = ((dstPitch * height) + bpp - 1) / bpp;
+   
    switch(id) {
    case FOURCC_YV12:
    case FOURCC_I420:
@@ -872,26 +882,25 @@ MGAPutImage(
 	break;
    }  
 
-   if(!(pPriv->area = MGAAllocateMemory(pScrn, pPriv->area, new_h)))
+   if(!(pPriv->linear = MGAAllocateMemory(pScrn, pPriv->linear, new_size)))
 	return BadAlloc;
 
     /* copy data */
-    top = y1 >> 16;
-    left = (x1 >> 16) & ~1;
-    npixels = ((((x2 + 0xffff) >> 16) + 1) & ~1) - left;
-    left <<= 1;
+   top = y1 >> 16;
+   left = (x1 >> 16) & ~1;
+   npixels = ((((x2 + 0xffff) >> 16) + 1) & ~1) - left;
+   left <<= 1;
 
-    offset = pPriv->area->box.y1 * pitch;
-    dst_start = pMga->FbStart + offset + left + (top * dstPitch);
+   offset = pPriv->linear->offset * bpp;
+   dst_start = pMga->FbStart + offset + left + (top * dstPitch);
 
-    if(pMga->TexturedVideo && pMga->AccelInfoRec->NeedToSync &&
+   if(pMga->TexturedVideo && pMga->AccelInfoRec->NeedToSync &&
 	((long)data != pPriv->lastPort)) 
-    {
+   {
 	MGAStormSync(pScrn);
-	pMga->AccelInfoRec->NeedToSync = FALSE;
-    }
+   }
 
-    switch(id) {
+   switch(id) {
     case FOURCC_YV12:
     case FOURCC_I420:
 	top &= ~1;
@@ -940,7 +949,8 @@ MGAPutImage(
 	     x1, y1, x2, y2, &dstBox, src_w, src_h, drw_w, drw_h);
 
 	pPriv->videoStatus = CLIENT_VIDEO_ON;
-    } 
+    }
+    pMga->VideoTimerCallback = MGAVideoTimerCallback;
 
     return Success;
 }
@@ -995,48 +1005,37 @@ MGAQueryImageAttributes(
 }
 
 static void
-MGABlockHandler (
-    int i,
-    pointer     blockData,
-    pointer     pTimeout,
-    pointer     pReadmask
-){
-    ScreenPtr      pScreen = screenInfo.screens[i];
-    ScrnInfoPtr    pScrn = xf86Screens[i];
-    MGAPtr         pMga = MGAPTR(pScrn);
+MGAVideoTimerCallback(ScrnInfoPtr pScrn, Time time)
+{
+    MGAPtr pMga = MGAPTR(pScrn);
     MGAPortPrivPtr pPriv = pMga->portPrivate;
 
-    pScreen->BlockHandler = pMga->BlockHandler;
-    
-    (*pScreen->BlockHandler) (i, blockData, pTimeout, pReadmask);
-
-    pScreen->BlockHandler = MGABlockHandler;
-
     if(pPriv->videoStatus & TIMER_MASK) {
-	UpdateCurrentTime();
 	if(pPriv->videoStatus & OFF_TIMER) {
-	    if(pPriv->offTime < currentTime.milliseconds) {
+	    if(pPriv->offTime < time) {
 		OUTREG(MGAREG_BESCTL, 0);
 		pPriv->videoStatus = FREE_TIMER;
-		pPriv->freeTime = currentTime.milliseconds + FREE_DELAY;
+		pPriv->freeTime = time + FREE_DELAY;
 	    }
 	} else {  /* FREE_TIMER */
-	    if(pPriv->freeTime < currentTime.milliseconds) {
-		if(pPriv->area) {
-		   xf86FreeOffscreenArea(pPriv->area);
-		   pPriv->area = NULL;
+	    if(pPriv->freeTime < time) {
+		if(pPriv->linear) {
+		   xf86FreeOffscreenLinear(pPriv->linear);
+		   pPriv->linear = NULL;
 		}
 		pPriv->videoStatus = 0;
+	        pMga->VideoTimerCallback = NULL;
 	    }
         }
-    }
+    } else  /* shouldn't get here */
+	pMga->VideoTimerCallback = NULL;
 }
 
 
 /****************** Offscreen stuff ***************/
 
 typedef struct {
-  FBAreaPtr area;
+  FBLinearPtr linear;
   Bool isOn;
 } OffscreenPrivRec, * OffscreenPrivPtr;
 
@@ -1048,8 +1047,8 @@ MGAAllocateSurface(
     unsigned short h,
     XF86SurfacePtr surface
 ){
-    FBAreaPtr area;
-    int pitch, fbpitch, numlines;
+    FBLinearPtr linear;
+    int pitch, fbpitch, size, bpp;
     OffscreenPrivPtr pPriv;
 
     if((w > 1024) || (h > 1024))
@@ -1057,38 +1056,39 @@ MGAAllocateSurface(
 
     w = (w + 1) & ~1;
     pitch = ((w << 1) + 15) & ~15;
-    fbpitch = pScrn->bitsPerPixel * pScrn->displayWidth >> 3;
-    numlines = ((pitch * h) + fbpitch - 1) / fbpitch;
+    bpp = pScrn->bitsPerPixel >> 3;
+    fbpitch = bpp * pScrn->displayWidth;
+    size = ((pitch * h) + bpp - 1) / bpp;
 
-    if(!(area = MGAAllocateMemory(pScrn, NULL, numlines)))
+    if(!(linear = MGAAllocateMemory(pScrn, NULL, size)))
 	return BadAlloc;
 
     surface->width = w;
     surface->height = h;
 
     if(!(surface->pitches = xalloc(sizeof(int)))) {
-	xf86FreeOffscreenArea(area);
+	xf86FreeOffscreenLinear(linear);
 	return BadAlloc;
     }
     if(!(surface->offsets = xalloc(sizeof(int)))) {
 	xfree(surface->pitches);
-	xf86FreeOffscreenArea(area);
+	xf86FreeOffscreenLinear(linear);
 	return BadAlloc;
     }
     if(!(pPriv = xalloc(sizeof(OffscreenPrivRec)))) {
 	xfree(surface->pitches);
 	xfree(surface->offsets);
-	xf86FreeOffscreenArea(area);
+	xf86FreeOffscreenLinear(linear);
 	return BadAlloc;
     }
 
-    pPriv->area = area;
+    pPriv->linear = linear;
     pPriv->isOn = FALSE;
 
     surface->pScrn = pScrn;
     surface->id = id;   
     surface->pitches[0] = pitch;
-    surface->offsets[0] = area->box.y1 * fbpitch;
+    surface->offsets[0] = linear->offset * bpp;
     surface->devPrivate.ptr = (pointer)pPriv;
 
     return Success;
@@ -1118,7 +1118,7 @@ MGAFreeSurface(
 
     if(pPriv->isOn)
 	MGAStopSurface(surface);
-    xf86FreeOffscreenArea(pPriv->area);
+    xf86FreeOffscreenLinear(pPriv->linear);
     xfree(surface->pitches);
     xfree(surface->offsets);
     xfree(surface->devPrivate.ptr);
@@ -1199,6 +1199,7 @@ MGADisplaySurface(
 	UpdateCurrentTime();
 	portPriv->videoStatus = FREE_TIMER;
 	portPriv->freeTime = currentTime.milliseconds + FREE_DELAY;
+	pMga->VideoTimerCallback = MGAVideoTimerCallback;
     }
 
     return Success;
